import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';

class ThemeProvider with ChangeNotifier {
  ThemeProvider({
    required this.darkTheme,
    required this.lightTheme,
    bool? dark,
  }) {
    final isDark = dark ?? _isDarkMode();
    switchTheme(isDark);
  }

  factory ThemeProvider.defaultThemes({Color? accentColor, Color? primaryColor}) {
    ThemeData darkTheme = ThemeData(
      accentColor: accentColor,
      brightness: Brightness.dark,
      primaryColor: primaryColor,
    );

    ThemeData lightTheme = ThemeData(
      accentColor: accentColor,
      brightness: Brightness.light,
      primaryColor: primaryColor,
    );
    return ThemeProvider(darkTheme: darkTheme, lightTheme: lightTheme);
  }

  ThemeData darkTheme;
  ThemeData lightTheme;

  ThemeData? _currentTheme;
  late ThemeMode _themeMode;

  ThemeData? get currentTheme => _currentTheme;

  ThemeMode get themeMode => _themeMode;

  bool get isDarkMode => _themeMode == ThemeMode.dark;

  switchTheme(bool? dark) async {
    if (dark == null) {
      _themeMode = ThemeMode.system;
      _currentTheme = null;
    } else {
      _themeMode = dark == true ? ThemeMode.dark : ThemeMode.light;
      _currentTheme = dark == true ? darkTheme : lightTheme;
    }
    notifyListeners();
  }

  bool? _isDarkMode() {
    var brightness = SchedulerBinding.instance?.window.platformBrightness;
    return brightness == Brightness.dark;
  }

  void changePrimaryColor(Color? color) {
    final newDarkTheme = darkTheme.copyWith(primaryColor: color);
    final newLightTheme = lightTheme.copyWith(primaryColor: color);
    darkTheme = newDarkTheme;
    lightTheme = newLightTheme;
    notifyListeners();
  }

  void changeAccentColor(Color? color) {
    final newDarkTheme = darkTheme.copyWith(accentColor: color);
    final newLightTheme = lightTheme.copyWith(accentColor: color);
    darkTheme = newDarkTheme;
    lightTheme = newLightTheme;
    notifyListeners();
  }
}
